;;;; e-w32.el --- MS Windows routines common to X and Gnu Emacs.
;;-*- Mode: Emacs-Lisp -*-   ;Comment that puts emacs into lisp mode
;;Time-stamp: <2002-05-15 13:09:37 W.U>
;;;;Outline-Mode: C-c @ C-t to show as index. C-c @ C-a to show all.
;;$Id$

;;__________________________________________________________________________
;;;;    TABLE OF CONTENTS
;;      Legal
;;      Commentary
;;      Code
;;              Display And Menu
;;              Wheel Mouse
;;              Printing Under Windows
;;              Ghostview Postscript
;;		Cygwin Find
;;              Replace Windows Command.Com/Cmd.Exe For Invoking "M-X Shell"
;;                      Z Shell
;;                      Cygwin Bash
;;                      Mks Korn Shell
;;              Windows  Fixes
;;                      Windows Command Shell Fixes
;;                      Win 95/98 Command.Com Fix
;;                      Stop Command Shell Echoing
;;                      Fix "Specified Command Search Directory Bad"
;;		Programming
;;              	Set Oracle For SQL Mode
;;			JDE BeanShell
;;		I18N Internationalization
;;              Internetworking
;;                      Windows Ftp
;;                      TCP/IP Dialup
;;                      Telnet
;;                      Web Browsing
;;              Woman: With Out Man(ual) Pages
;;              Outline-mode control variables

;;__________________________________________________________________________
;;; Legal:
;;Copyright  1998,2002 Bruce Ingalls bingalls@sf.net
;;This file is not (yet) part of Emacs nor XEmacs. See file COPYING for license

;;__________________________________________________________________________
;;;; Commentary:
;;Emacs editor startup file.  MS Windows routines common to X & Gnu Emacs

;;__________________________________________________________________________
;;; Author:		Bruce Ingalls <bingalls@panix.com>
;;<url:ftp://ftp.cppsig.org/pub/tools/emacs/>	<url:http://www.ikoch.de/>
;;; Change Log:		See ChangeLog history file
;;;; Keywords:		microsoft windows w32 w2k win32

;;__________________________________________________________________________
;;;; Code:
(autoload 'outline-minor-mode "outline" "automagic index to jump to" t)
(defgroup e-w32 nil "Settings from e-w32.el file." :group 'emacro)
(require 'dired)

(autoload 'gud "gud" "gud debug files." t)
(autoload 'perldb "gud" "debug perl files." t)
(autoload 'comint-send-input "comint" "comint." t)

;;__________________________________________________________________________
;;;;    Display And Menu
;;(set-message-beep 'ok)                 ;"OK" windows system sound

;;Use the alt key as alt, not meta:
;;(setq win32-alt-is-meta nil)  ; Emacs 19 (XEmacs?)
;;(setq w32-alt-is-meta nil)    ; Emacs 20

;;__________________________________________________________________________
;;;;    Wheel Mouse
;; (defun mouse-wheel-handler (event)
;;   "Scroll the current buffer by 'mouse-wheel-scroll-amount'."
;;   (interactive "e")
;;   (condition-case nil
;;       (if (< (car (cdr (cdr event))) 0)
;;         (scroll-up mouse-wheel-scroll-amount)
;;       (scroll-down mouse-wheel-scroll-amount))
;;     (error nil)))
;; ;; Fix wierd shell problem.
;; (add-hook 'sh-mode-hook
;;         (defun my-shell-mode-hook ()
;;           (sh-set-shell
;;            "c:/usr/local/cygnus/cygwin-b20/H-i586-cygwin32/bin/bash")))


;;__________________________________________________________________________
;;;;    Printing Under Windows
;;See also printer section in poweruser.el, preferences.el and solutions.html
;;Example printer name:
;;(defconst printer-name "\\\\server-name\\printer-port-name")
(autoload 'ps-print-buffer "ps-print" "postscript print" t)
(autoload 'ps-print-region "ps-print" "postscript print" t)

(cache-locate-library
 use-cache 'cache-lpr "lpr"
 "For printing support, get lpr.el from full XEmacs sumo tarball")
(when cache-lpr (require 'lpr))

(defcustom lpr-command "print" "Print command for w32."
  :group 'e-w32
  :type 'string)
;;(defconst ps-lpr-command "print")

;;If you grab <url:http://hem1.passagen.se/ptlerup> uncomment the following
;;printfile.exe seems to automagically determine if it's postscript
;;(defconst printer-name nil)
;;(defconst lpr-command "c:/progra~1/printfile/prfile32.exe")
;;(defconst lpr-switches '("/q"))

;;Alternate solution at <url: http://www.lerup.com/printfile/ >
;; (setq ps-lpr-command "d:/programmer/printfile/prfile.exe")
;; (setq ps-printer-name t)

;; (setq printer-name nil)
;; (setq lpr-command "d:/programmer/printfile/prfile.exe")

;;__________________________________________________________________________
;;;;    Ghostview Postscript
;;This should allow non postscript printers to print
(defvar cache-ps-print)			;shut up compiler
(cache-locate-library
 use-cache 'cache-ps-print "ps-print"
 "For postscript printing support, Get ps-print from full XEmacs sumo tarball")
(when cache-ps-print (require 'ps-print))

(defvar use-paper)			;shut up compiler
(defconst ps-paper-type use-paper)
(defcustom ps-print-color-p nil "Toggle color postscript."
  :group 'e-w32
  :type 'boolean)

(defcustom ps-header-lines 1 "Postscript header size."
  :group 'e-w32
  :type 'integer)
(defcustom ps-print-header-frame nil "Toggle postscript headers."
  :group 'e-w32
  :type 'boolean)
(defcustom ps-print-only-one-header t "Toggle postscript headers."
  :group 'e-w32
  :type 'boolean)

;;Assume that w32 users with ghostview in path, use it to print.
;;ToDo: probably need to prompt users in future.
(when (which "ghostview")
  (defcustom ghostview-program (which-first "ghostview") "Postscript program."
    :group 'e-w32
    :type 'string)
  (defcustom ps-printer-name t "Toggle if printer is postscript."
    :group 'e-w32
    :type 'boolean)
  (defcustom (defconst ps-lpr-switches '(ghostview-program))
    "Postscript program flags."
    :group 'e-w32
    :type 'string)
  (defcustom ps-lpr-command "cmdproxy" "Shell for postscript program."
    :group 'e-w32
    :type 'string))

;;"snapshot" is caught by the system by default, so register it as a hotkey.
;;w32-register-hot-key is a Gnu Emacs (only) builtin.
(when (string-match "GNU" (emacs-version)) (w32-register-hot-key [snapshot]))

(define-key global-map [snapshot] 'ps-print-buffer-with-faces)
;; BDF font locations for ps-bdf (printing) and w32bdf (display)
(defconst bdf-directory-list
  '("c:/fonts/Asian" "c:/fonts/Chinese" "c:/fonts/Chinese-X"
    "c:/fonts/Ethiopic" "c:/fonts/European" "c:/fonts/Japanese-X"
    "c:/fonts/Japanese" "c:/fonts/Korean-X" "c:/fonts/Misc"))

;;__________________________________________________________________________
;;;;;;          Cygwin Find
;;Enables recursive grep and prevents bytecompilation crash
;;I recommend you stay with Cygnus's default path!
(defcustom igrep-find-program "/bin/find"
  "Full path to Cygnus find.exe, not Microsoft find.exe."
  :group 'e-w32
  :type 'string)

;;__________________________________________________________________________
;;;;    Replace Windows Command.Com/Cmd.Exe For Invoking "M-X shell"
;;You need to get the executable for one of the shells and uncomment the
;;appropriate block

;;__________________________________________________________________________
;;;;;;          Z Shell         ftp://ftp.blarg.net/users/amol/zsh
;;consider creating ~/.zshenv startup with 'TERM=emacs exec zsh' (see zsh faq)
;;	Begin Shell Detection Block
(defvar explicit-shell-file-name)	;shut up compiler
(cond
 ((which "zsh")
  (defcustom use-sh "zsh" "Shell to invoke from Emacs."
    :group 'e-w32
    :type 'string))
 ((which "bash")
  (defcustom use-sh "bash" "Shell to invoke from Emacs."
    :group 'e-w32
    :type 'string))
 ((which "ksh")
  (defcustom use-sh "ksh" "Shell to invoke from Emacs."
    :group 'e-w32
    :type 'string))
 (t (defvar use-sh nil)))		;else nil for no shell in PATH

(when use-sh				;if a shell is found in PATH
  (cond
   ((string= use-sh "zsh")
    (defcustom explicit-shell-file-name (which-first "zsh") "Full zsh path."
      :group 'e-w32
      :type 'string)
    (defcustom exec-path (cons explicit-shell-file-name exec-path)
      "Full path to shell."
      :group 'e-w32
      :type 'string)
    (defcustom shell-file-name explicit-shell-file-name "Full path to shell."
      :group 'e-w32
      :type 'string)
    (defcustom sh-shell-file explicit-shell-file-name "Full path to shell."
      :group 'e-w32
      :type 'string)
    (defcustom shell-command-switch "-c" "Flags to pass to shell."
      :group 'e-w32
      :type 'string)
    (defcustom win32-quote-process-args t "Shell toggle. For advanced users."
      :group 'e-w32
      :type 'boolean)
    (defcustom zsh-dir explicit-shell-file-name "Full path to zsh."
      :group 'e-w32
      :type 'string)
    (setenv "ESHELL" explicit-shell-file-name)
    (add-hook 'comint-output-filter-functions 'shell-strip-ctrl-m nil t)
    (add-hook 'shell-mode-hook
	      '(lambda () (setq comint-completion-addsuffix t)) t))

;;__________________________________________________________________________
;;;;;;          Cygwin Bash             http://www.cygnus.com/
;;Else

   ((string= use-sh "bash")
    (defcustom explicit-shell-file-name (which-first "bash") "Full bash path."
      :group 'e-w32
      :type 'string)
    (defcustom exec-path (cons explicit-shell-file-name exec-path)
      "Full path to shell."
      :group 'e-w32
      :type 'string)
    (defcustom explicit-sh-args '("-login" "-i") "Flags to pass to shell."
      :group 'e-w32
      :type 'string)
    (defcustom sh-shell-file explicit-shell-file-name "Full path to shell."
      :group 'e-w32
      :type 'string)
    (defcustom shell-file-name explicit-shell-file-name "Full path to shell."
      :group 'e-w32
      :type 'string)
    (defcustom shell-command-switch "-c" "Flags to pass to shell."
      :group 'e-w32
      :type 'string)
    (defcustom win32-quote-process-args t "Shell toggle. For advanced users."
      :group 'e-w32
      :type 'boolean)
    (setenv "ESHELL" shell-file-name)
    (add-hook 'comint-output-filter-functions 'shell-strip-ctrl-m nil t)

    (setenv "SHELL" shell-file-name)

    ;;getenv() is not available in all emacs
    ;;  (setenv "PATH" (concat (concat shell-file-name ";") (getenv "PATH")))

    (add-hook 'shell-mode-hook
	      '(lambda () (setq comint-completion-addsuffix t)) t))

;;__________________________________________________________________________
;;;;;;          MKS Korn Shell          http://www.mks.com/
;;Else

   ((string= use-sh "ksh")
    (defcustom explicit-shell-file-name (which-first "ksh") "Full ksh path."
      :group 'e-w32
      :type 'string)
    (defcustom exec-path (cons explicit-shell-file-name exec-path)
      :group 'e-w32
      :type 'string)
    (defcustom shell-file-name explicit-shell-file-name "Full path to shell."
      :group 'e-w32
      :type 'string)
    (defcustom sh-shell-file explicit-shell-file-name "Full path to shell."
      :group 'e-w32
      :type 'string)
    (defcustom shell-command-switch "-c" "Flags to pass to shell."
      :group 'e-w32
      :type 'string)
    (defcustom win32-quote-process-args t "Shell toggle. For advanced users."
      :group 'e-w32
      :type 'boolean)

    (setenv "ESHELL" (which-first "ksh"))
    (add-hook 'comint-output-filter-functions 'shell-strip-ctrl-m nil t)
    (add-hook 'shell-mode-hook
	      '(lambda () (setq comint-completion-addsuffix t)) t))))

;;	End Shell Detection Block

;;__________________________________________________________________________
;;;;    Windows  Fixes
;;add these if the programs mentioned are't working.
;;(setq grep-null-device null-device)                    ;igrep 2.82
;;(let ((emacs-minor-version 2)) (require 'lazy-lock))   ;20.3 lazy lock
;;(setq backup-by-copying t) ;stop single original overwriting backup

;;__________________________________________________________________________
;;;;            Windows Command Shell Fixes
;;This strips ^Ms in command shell mode:
;;(add-hook 'comint-output-filter-functions 'shell-strip-ctrl-m nil t)

;;__________________________________________________________________________
;;;;;;          Windows 95/98 Command.Com Fix
;;(setq process-coding-system-alist
;;      '(("cmdproxy" . (raw-text-dos . raw-text-dos))))

;;__________________________________________________________________________
;;;;;;          Stop Command Shell Echoing
;;FIRST, TRY:
;;(defun my-comint-init ()
;;  (setq comint-process-echoes t))
;;(add-hook 'comint-mode-hook 'my-comint-init)
;;THEN TRY:
;;(setq explicit-cmd.exe-args '("/q"))

;;__________________________________________________________________________
;;;;;;          Fix "Specified Command Search Directory Bad"
;;FIRST TRY:
;;(load-library "winnt")
;;THEN TRY:
;;windows 95/98
;;(setq explicit-command.com-args nil)
;;(setq explicit-COMMAND.COM-args nil)
;;windows nt
;;(setq explicit-cmd.exe-args nil)
;;(setq explicit-CMD.EXE-args nil)

;;__________________________________________________________________________
;;;;;;    Set Oracle For SQL Mode
;;See Also Sql-Mode In programmer.el !
;;sql.el:
(defconst sql-oracle-program "n")
(when (which "PLUS33")			;Oracle 7.x
  (defcustom sql-oracle-program (which-first "PLUS33") "Path to Oracle."
    :group 'e-w32
    :type 'string))
(when (which "PLUS80")			;Oracle 8.x
  (defcustom sql-oracle-program (which-first "PLUS80") "Path to Oracle."
    :group 'e-w32
    :type 'string))
;;sql-mode.el:  http://geek.stanford.edu
(when (not (string= sql-oracle-program "n"))
  (defcustom sql-process-name sql-oracle-program "SQL interpreter name."
    :group 'e-w32
    :type 'string)
  (defcustom sql-command sql-oracle-program "SQL interpreter name."
    :group 'e-w32
    :type 'string)

;;sqlplus-mode.el: http://www.anc.ed.ac.uk/~stephen/emacs/ell.html
  (defcustom sqlplus-command sql-oracle-program "SQL interpreter name."
    :group 'e-w32
    :type 'string)
  (defvar sql-user)			;shut up compiler
  (defvar sql-database)
  (defcustom sqlplus-command-args (format "%s@%s" sql-user sql-database)
    "Oracle login default. For less security, try 'name/password@sid'."
    :group 'e-w32
    :type 'string))

;;Bugs: Oracle limits buffers pasted in to about 250 lines

;;__________________________________________________________________________
;;;;;;    JDE BeanShell
;;This likely needs work...

;;From: crabb@phoenix.Princeton.EDU (David W. Crabb)
;;Although I use XEmacs/JDK/JDE on Windows98, the following might at least
;;give those using other platforms some hints as to how to get 'bsh' operative
;;in XEmacs. Note that "hard-coded" classpath fed to function 'comint-exec'
;;below must be adjusted for each implementation of JDE within XEmacs
  
;; (defun my-bsh ()
;;   " A \"hard-coded\" core of function 'bsh-internal', which can replace that function."
;;   (interactive)
;;   (let* ( (bsh-buffer-name "*bsh*")
;;           (bsh-buffer (get-buffer-create bsh-buffer-name)) )
;;     (save-excursion
;;       (set-buffer bsh-buffer)
;;       (erase-buffer)
;;       (comint-mode))
;;     (save-w32-show-window
;;      (message "%s" "Starting the BeanShell. Please wait...")
;;      (comint-exec "*bsh*" "bsh" "c:/Progra~1/JavaSoft/JRE/1.2/bin/java" nil '( "-classpath" "C:/PROGRA~1/XEMACS/xemacs-packages/etc/jde/java/lib/jde.jar;C:/PROGRA~1/XEMACS/xemacs-packages/etc/jde/java/lib/bsh.jar" "bsh.Interpreter") )
;;      (pop-to-buffer bsh-buffer-name)  )) )

;;;(fset 'bsh 'my-bsh)	;if it is desired to put this version in the .emacs initfile
;;;and run from other than the JDE menubutton .

;;;PLUS a modification to beanshell.el, byte-compiled into a fresh beanshell.elc :

;; (defun bsh()
;;   "*Starts BeanShell, a Java interpreter developed by Pat Niemeyer."
;;   (interactive)
;;   ;;DWCmod  (bsh-internal t))
;;   (my-bsh))

;;__________________________________________________________________________
;;;;    I18N Internationalization
;;(setq w32-enable-unicode-output nil)

;;(setq w32-enable-italics t)
;;(create-fontset-from-fontset-spec
;;  "-*-Courier New-normal-r-*-*-*-120-*-*-c-*-fontset-courier,
;;   ascii:-*-Courier New-normal-r-*-*-*-120-*-*-c-*-iso8859-1,
;;   latin-iso8859-1:-*-Courier New-normal-r-*-*-*-120-*-*-c-*-iso8859-1,
;;   latin-iso8859-2:-*-Courier New-normal-r-*-*-*-120-*-*-c-*-iso8859-2,
;;   latin-iso8859-3:-*-Courier New-normal-r-*-*-*-120-*-*-c-*-iso8859-3,
;;   latin-iso8859-4:-*-Courier New-normal-r-*-*-*-120-*-*-c-*-iso8859-4,
;;   cyrillic-iso8859-5:-*-Courier New-normal-r-*-*-*-120-*-*-c-*-iso8859-5,
;;   greek-iso8859-7:-*-Courier New-normal-r-*-*-*-120-*-*-c-*-iso8859-7,
;;   latin-iso8859-9:-*-Courier New-normal-r-*-*-*-120-*-*-c-*-iso8859-9"
;; t)

;;(setq default-frame-alist
;;      '((font . "-*-Courier New-normal-r-*-*-*-120-*-*-c-*-fontset-courier")))

;;__________________________________________________________________________
;;;;    Internetworking
;;__________________________________________________________________________
;;;;;;          Windows FTP
;;allows editing remote files with syntax:
;;      C-x C-f         /username@host:/home/file.txt
;;(defconst ange-ftp-ftp-program-name "c:/winnt/system32/ftp.exe") ;path to ftp
;;(defconst ange-ftp-tmp-name-template              ;;where to cache ftp files
;;      (concat (expand-file-name (getenv "TEMP")) "/ange-ftp"))
;;(defconst ange-ftp-gateway-tmp-name-template
;;      (concat (expand-file-name (getenv "TEMP")) "/ange-ftp"))

;;__________________________________________________________________________
;;;;;;          TCP/IP Dialup
;;If emacs prompts you for a dial-up connection, get nttcp.exe & tcp.el:
;;http://www.cs.washington.edu/homes/voelker/ntemacs/contrib/nttcp.exe
;;http://www.cs.washington.edu/homes/voelker/ntemacs/contrib/tcp.el
;;then add:
;;(load-library "tcp")
;;(defconst tcp-program-name "nttcp")

;;__________________________________________________________________________
;;;;;;          Telnet
;;See also poweruser.el and xemacs.el
;;This works in XEmacs, if you type it in by hand, but doesn't autoload from
;; .emacs. Try ftelnet() from xemacs.el, instead.

;;You need a _Console_ telnet program, like utilities.zip from
;; <URL:http://hesweb1.med.virginia.edu/biostat/s/EmacsTeX/index.html>
;;telnet-mode for w32 also from
;; <URL:http://www.cs.washington.edu/homes/voelker/ntemacs/contrib/>
;;This seems to require telnet.el bundled with emacs
(autoload 'telnet "telnet" "telnet" t)
(autoload 'telnet-mode "telnet" "telnet" t)

(defcustom telnet-program (which-first "telnet")
  "Full path to telnet.exe. Avoid Microsoft's telnet."
  :group 'e-w32
  :type 'string)

;;__________________________________________________________________________
;;;;	Tiny Tools
;;Lets you click urls in text to lauch browser
(defvar cache-tinyurl)
(cache-locate-library
 use-cache 'cache-advice "advice"
 "get advice.el from full sumo tarball distribution where you got XEmacs")
(cache-locate-library
 use-cache 'cache-tinyurl "tinyurl"
 "get tiny-tools from <URL:http://poboxes.com/jari.aalto/emacs-tiny-tools.html>")
(defvar use-modem)			;shut up compiler
(when (and cache-advice cache-tinyurl)
  (if (which "shellex")
      (progn
	(add-hook 'tinyurl-:load-hook  'tinyurl-install-to-packages)
        (autoload 'tinyurl-mode "tinyurl" "click urls to lauch browser" t)
;;ToDo: remove this duplicate code:
;;	(autoload 'tinyurl-mode-1      "tinyurl" t t)
	(tinyurl-mode)
	(cond ((and (boundp use-modem) (not use-modem))
	       (defconst tinyurl-:plugged-function '(lambda () t))))
	))                        ;else no shellex.exe
  (message "Put shellex.exe to PATH, from http://www.tertius.com/projects/library/"))


;;__________________________________________________________________________
;;;;	Woman: With Out Man(ual) Pages
;;Get woman.el from http://www.anc.ed.ac.uk/~stephen/emacs/ell.html
(autoload 'define-key-after "subr" "subr.el" t)
(defvar cache-woman)			;shut up compiler
(cache-locate-library
 use-cache 'cache-woman "woman"
 "For Manual pages, get woman from <url:http://centaur.maths.qmw.ac.uk/Emacs/>")

(when cache-woman
  (autoload 'woman "woman" "Decode and browse a UN*X man page." t)
  (autoload 'woman-find-file "woman"  "Find, decode & browse a  man-page." t)
  ;;(global-set-key [f6] 'woman-find-file)
  (autoload 'woman-dired-find-file "woman"
    "In dired, run the WoMan man-page browser on this file." t)
  (add-hook 'dired-mode-hook
            (function
             (lambda ()
               (define-key dired-mode-map "W" 'woman-dired-find-file))))
  
  (let (help-menu manuals)
    (if (setq help-menu (lookup-key global-map [menu-bar help-menu]))
        (if (setq manuals (lookup-key help-menu [manuals]))
            (define-key-after
              manuals [woman] '("Read Man Page (WoMan)..." . woman) t)
          (define-key-after
            help-menu [woman] '("WoMan..." . woman) 'man))))

  (defcustom woman-always-colour-faces t "Use color instead of bold & underscore."
    :group 'e-w32
    :type 'boolean)
  (defcustom woman-fill-frame t "Fill up the full width of the frame."
    :group 'e-w32
    :type 'boolean))

;;__________________________________________________________________________
;;;;	Misc
;;'M-x w32-faq' optional package for common help
;;(require 'w32-shellex	;;http://www.tertius.com/projects/library/index.html
;;(require 'w32-faq)	;<url:http://GNUSoftware.com/Emacs/Lisp/>

;;__________________________________________________________________________
;;;;    Outline-mode control variables
;;4 ';'s at beginning of line makes an index entry
;;more ';'s nests the entry deeper

;; Local Variables:
;; mode: outline-minor
;; center-line-decoration-char: ?-
;; center-line-padding-length: 1
;; center-line-leader: ";;;; "
;; fill-column: 79
;; line-move-ignore-invisible: t
;; outline-regexp: ";;;;+"
;; page-delimiter: "^;;;;"
;; End:
(provide 'e-w32)

;;; e-w32.el ends here
