;;;; e-java.el --- Java support for gnu and x emacs for every OS.
;;-*- Mode: Emacs-Lisp -*-   ;Comment that puts emacs into lisp mode
;;Time-stamp: <2002-03-05 23:46:23 bingalls>
;;;;Outline-Mode: C-c @ C-t to show as index. C-c @ C-a to show all.
;;$Id$

;;__________________________________________________________________________
;;;;	TABLE OF CONTENTS
;; 	Legal
;; 	Commentary
;; 	Code
;;	Jikes javac compiler
;;	Java Hooks
;;	JDE Java Development Environment
;;	Java Accessories
;;	Outline-mode control variables

;;__________________________________________________________________________
;;; Legal:
;;Copyright  1998,2002 Bruce Ingalls bingalls@sf.net
;;This file is not (yet) part of Emacs nor XEmacs. See file COPYING for license

;;__________________________________________________________________________
;;; Commentary:
;;Emacs editor startup file. Java support for every OS.

;;__________________________________________________________________________
;;; Author:		Bruce Ingalls <bingalls@panix.com>
;;<url:http://www.sourceforge.net/projects/emacro>
;;<url:http://www.dotemacs.de/>
;;; Change Log:		See ChangeLog history file
;;; Keywords:		program mode ide

;;__________________________________________________________________________
;;;; Code:
(autoload 'outline-minor-mode "outline" "automagic index to jump to" t)
(defgroup e-java nil "Settings from e-java.el file." :group 'emacro)
(require 'which)

;;__________________________________________________________________________
;;;;		Jikes javac compiler
;;Check for jikes in $PATH, which is faster than javac
;;<URL:http://ibm.com/developerworks/opensource/>

;;DEPRECATE these comments
;;(defconst jde-compiler "javac")
;;(defconst jde-compile-option-command-line-args "")

;;(if (which "jikes")
;;    (progn

;;DEPRECATED code. Use it, if your JDE version is old
;;      (setq jde-compiler "jikes")

;;      (setq jde-compiler '("jikes" ""))
;;      (setq jde-interactive-compile-args  "+E +P -deprecation -depend")
;;      )                                 ;else no jikes in path
;;  (message
;;   "For a fast javac compiler support, get jikes from <url:http://oss.software.ibm.com/developerworks/opensource/jikes/project/index.html>"))

;;__________________________________________________________________________
;;;;		Java Hooks
;;(setq tags-file-name "/edit/this/dir")	;find function declares & refs
(defcustom java-font-lock t "Enable Java syntax coloring."
  :group 'e-java
  :type 'boolean)

;;(defvar jde-compiler)
;;(defvar jde-compile-option-command-line-args)

(autoload 'qflib-mode "qflib" "Minor mode utilities for editing Java code")

(cache-locate-library
 use-cache 'cache-jde "jde"
 "For a Java IDE, get JDE from <url:http://sunsite.auc.dk/jde/>")

(when cache-jde
  (add-hook 'jde-mode-hook
            '(lambda ()
               (local-set-key [?\C-c ?\C-c] 'jde-complete-at-point-menu)
	       (font-lock-fontify-buffer)
	       )))
;;DEPRECATE this code
;;               (setq jde-compiler use-jde-compiler)
;;               (setq jde-compile-option-command-line-args
;;                     use-jde-compile-option-command-line-args))))

(cache-locate-library
 use-cache 'cache-qflib "qflib"
 "For a Java menu, get qflib from <url:http://www.qfs.de/en/projects/elisp/index.html>")

(defvar use-indent)			;shut up compiler
;;Java-mode-hook also gets called by JDE, affecting both
(add-hook 'java-mode-hook
          '(lambda ()
             (setq c-file-style "local")
             (setq c-basic-offset use-indent)
             ;;(setq fill-column '79)	;word-wrap your code
             ;;(turn-on-auto-fill)		;word-wrap your code

             (when cache-qflib
               (defconst qflib-prefix-key "\C-x\C-j")
               (global-set-key "\C-x\C-j\C-m" 'qflib-find-stacktrace-error)
               ;;(setq qflib-source-directories 
               ;;      (list (expand-file-name "~/src/Java")))
               (qflib-mode))))

;;__________________________________________________________________________
;;;;		JDE: Java Development Environment
;;Get JDE from <URL:http://sunsite.auc.dk/jde/> See JDE section below
;;Support for gnu emacs appears to be less than for xemacs

;;Prevent browser (launched by jde help) from locking emacs:
;;(defun shell-execute-url (url &optional new-window)
;;  "Invoke the shell-execute-helper program to call ShellExecute and launch
;;          or re-direct a web browser on the specified url."
;;  (interactive "sURL: ")
;;  (call-process shell-execute-helper nil nil nil url)
;;  (start-process "shellex" nil shell-execute-helper url))

(cache-locate-library
 use-cache 'cache-speedbar "speedbar"
 "For fast navigation menu, get speedbar.el from full XEmacs sumo tarball")

(cond
 ((and cache-jde cache-speedbar)
  (autoload 'jde-mode "jde" "Java Development Environment" t)
  (setq auto-mode-alist (append '(
                                  ("\\.java$" . jde-mode)
                                  ("\\.jsp$"  . jde-mode)
                                  ) auto-mode-alist))
  ;;(defconst jde-jdk-doc-url "/jdk1.2.2/readme.html")

  );;Else use plain java mode:
 (t (setq auto-mode-alist(append '(
				   ("\\.java$" . java-mode)
				   ("\\.jsp$"  . java-mode)
				   )auto-mode-alist))))

;;__________________________________________________________________________
;;;;		Java Accessories
(cache-locate-library
 use-cache 'cache-java-open "java-open"
 "For S-f10 over class name to open its file, get java-open.el from http://www.mayura.com/misc/")

;;S-f10 over a class name opens that file, if you set CLASSPATH
(when (and cache-java-open (getenv "CLASSPATH"))
  (defvar java-open-source-path)	;shut up compiler
  (require 'java-open)
  (global-set-key [(f10)] 'java-open-class-at-point)
  (global-set-key [(f11)] 'java-open-base-class-source)

  ;;Find java source in CLASSPATH. Benjamin Rutt <brutt@bloomington.in.us>
  (cond ((or (eq "ms-dos" system-type)
             (string-match "windows" (format "%s" system-type)))
         (setq java-open-source-path
               (delq nil
                     (mapcar (lambda (x) (if (file-directory-p x) x nil))
                             (split-string (getenv "CLASSPATH") ";")))))
        (t                              ;Linux (mac?) separate paths with ":"
         (setq java-open-source-path
               (delq nil
                     (mapcar (lambda (x) (if (file-directory-p x) x nil))
                             (split-string (getenv "CLASSPATH") ":")))))))


;;SET THIS IN HOME.EL!
;;source-path is similar in function to CLASSPATH
;;      (defconst source-path '("D:/java/source" "C:/jdk1.3/src"))

;;__________________________________________________________________________
;;;;	Outline-mode control variables
;;4 ';'s at beginning of line makes an index entry
;;more ';'s nests the entry deeper

;; Local Variables:
;; mode: outline-minor
;; center-line-decoration-char: ?-
;; center-line-padding-length: 1
;; center-line-leader: ";;;; "
;; fill-column: 79
;; line-move-ignore-invisible: t
;; outline-regexp: ";;;;+"
;; page-delimiter: "^;;;;"
;; End:

(provide 'e-java)
;;; e-java.el ends here
